#!/opt/concourse/worker/volumes/live/1607423b-5b29-462f-51f4-7d9f9ff7726c/volume/django_1636039297968/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehol/bin/python
from django.core import management

if __name__ == "__main__":
    management.execute_from_command_line()
